module LoadedDie
  VERSION = '1.0.1'

  class Sampler
    DEFAULT_RNG = ::Object.new
    def DEFAULT_RNG.rand(n)
      ::Kernel.rand * n
    end

    Segment = ::Struct.new(:maximum, :individual)

    def initialize(population)
      @compiled = population.inject [] do |accum, (individual, weight)|
        if weight <= 0
          raise ::ArgumentError, "non-positive weight #{weight}"
        end
        prev_max =
          if last = accum.last
            last.maximum
          else
            0
          end
        accum << Segment.new(prev_max + weight, individual)
      end
      nil
    end

    def length
      if last = @compiled.last
        last.maximum
      else
        0
      end
    end

    def [](point)
      if point < 0
        nil
      elsif choice = @compiled.detect { |segment| point < segment.maximum }
        choice.individual
      else
        nil
      end
    end

    def sample(options = {})
      rng = options.fetch(:random) { DEFAULT_RNG }
      point = rng.rand(length)
      self[point]
    end
  end
end
